@@ -1,5 +1,6 @@
 import abc
 from typing import List
+from contextlib import contextmanager
 from uuid import UUID
 from shared_context.domain.model import AggregateRoot, Repository as BaseRepository
 from sqlalchemy import create_engine
@@ -8,6 +9,23 @@ from sqlalchemy.dialects.mysql import BINARY
 from sqlalchemy.types import TypeDecorator
 
 
+@contextmanager
+def session_scope(dsn: str):
+    Session = sessionmaker(bind=create_engine(dsn))
+
+    session = Session()
+    try:
+        yield session
+
+        session.commit()
+    except Exception:
+        session.rollback()
+
+        raise
+    finally:
+        session.close()
+
+
 class SessionBuilder:
     _session = None
 
@@ -23,25 +41,31 @@ class SessionBuilder:
 
 
 class Repository(BaseRepository):
-
     def __init__(self, aggregate: AggregateRoot, dsn: str):
         super().__init__(aggregate)
 
-        self.__session = SessionBuilder.build(dsn)
+        self._dsn = dsn
+        # self.__session = SessionBuilder.build(dsn)
 
     def add(self, aggregate: AggregateRoot) -> None:
-        self.__session.add(aggregate)
-        self.__session.commit()
+        with session_scope(self._dsn) as session:
+            session.add(aggregate)
 
     def save(self, aggregate: AggregateRoot) -> None:
-        self.__session.add(aggregate)
-        self.__session.commit()
+        with session_scope(self._dsn) as session:
+            session.add(aggregate)
 
     def find(self, **kwargs) -> AggregateRoot:
-        return self.__session.query(self.__aggregate).filter_by(**kwargs).first()
+        with session_scope(self._dsn) as session:
+            result = session.query(self.__aggregate).filter_by(**kwargs).first()
+
+        return result
 
     def find_all(self, **kwargs) -> List[AggregateRoot]:
-        return self.__session.query(self.__aggregate).filter_by(**kwargs).all()
+        with session_scope(self._dsn) as session:
+            results = session.query(self.__aggregate).filter_by(**kwargs).all()
+
+        return results
 
 
 class Orm(metaclass=abc.ABCMeta):
